Class: OpenStudio::Analysis::Workflow

Inherits:
Object
  • Object
show all
Defined in:
lib/openstudio/analysis/workflow.rb

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize ⇒ Object

Create an instance of the OpenStudio::Analysis::Workflow



12
13
14
# File 'lib/openstudio/analysis/workflow.rb', line 12

def initialize
  @items = []
end

Instance Attribute Details

#items ⇒ Object (readonly) Also known as: measures

Returns the value of attribute items.



5
6
7
# File 'lib/openstudio/analysis/workflow.rb', line 5

def items
  @items
end

Class Method Details

.from_file(filename) ⇒ Object

Read the Workflow description from a persisted file. The format at the moment is the current analysis.json

Returns:

  • (Object) —

    Return an instance of the workflow object



182
183
184
185
186
187
188
189
190
191
192
193
194
195
196
197
198
# File 'lib/openstudio/analysis/workflow.rb', line 182

def self.from_file(filename)
  if File.exist? filename
    j = JSON.parse(File.read(filename), symbolize_names: true)

    # get the version of the file
    file_format_version = j[:file_format_version] ? j[:file_format_version] : 1

    puts "Parsing file version #{file_format_version}"

  else
    fail "Could not find workflow file #{filename}"
  end

  o = OpenStudio::Analysis::Workflow.new
  # put the JSON into the right format
  o
end

Instance Method Details

#add_measure(instance_name, instance_display_name, path_to_measure, measure_metadata) ⇒ Object

Add a measure from the custom hash format without reading the measure.rb or measure.json file

Parameters:

  • path_to_measure (String) —

    This is the local path to the measure directory, relative or absolute. It is used when zipping up all the measures.

  • measure_metadata (Hash) —

    Format of the measure.json

Returns:

  • (Object) —

    Returns the measure that was added as an OpenStudio::AnalysisWorkflowStep object



66
67
68
69
70
# File 'lib/openstudio/analysis/workflow.rb', line 66

def add_measure(instance_name, instance_display_name, path_to_measure, )
  @items << OpenStudio::Analysis::WorkflowStep.from_measure_hash(instance_name, instance_display_name, path_to_measure, )

  @items.last
end

#add_measure_from_excel(measure) ⇒ Object

Add a measure from the format that Excel parses into. This is a helper method to map the excel data to the new programmatic interface format

Returns:

  • (Object) —

    Returns the measure that was added as an OpenStudio::AnalysisWorkflowStep object



77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
# File 'lib/openstudio/analysis/workflow.rb', line 77

def add_measure_from_excel(measure)
  hash = {}
  hash[:classname] = measure['measure_file_name']
  hash[:name] = measure['name']
  hash[:display_name] = measure['display_name']
  hash[:measure_type] = measure['measure_type']
  hash[:uid] = measure['uid'] ? measure['uid'] : SecureRandom.uuid
  hash[:version_id] = measure['version_id'] ? measure['version_id'] : SecureRandom.uuid

  # map the arguments - this can be a variable or argument, add them all as arguments first
  args = []
  measure['variables'].each do |variable|
    args << {
      local_variable: variable['name'],
      variable_type: variable['type'],
      name: variable['name'],
      display_name: variable['display_name'],
      display_name_short: variable['display_name_short'],
      units: variable['units'],
      default_value: variable['distribution']['static_value'],
      value: variable['distribution']['static_value']
    }
  end
  hash[:arguments] = args

  m = add_measure(measure['name'], measure['display_name'], measure['measure_file_name_directory'], hash)

  measure['variables'].each do |variable|
    next unless variable['variable_type'] == 'variable'

    dist = {
      type: variable['distribution']['type'],
      minimum: variable['distribution']['min'],
      maximum: variable['distribution']['max'],
      mean: variable['distribution']['mean'],
      standard_deviation: variable['distribution']['stddev'],
      values: variable['distribution']['discrete_values'],
      weights: variable['distribution']['discrete_weights'],
      step_size: variable['distribution']['delta_x']
    }
    opt = {
      variable_type: variable['variable_type'],
      variable_display_name_short: variable['display_name_short'],
      static_value: variable['distribution']['static_value']
    }

    m.make_variable(variable['name'], variable['display_name'], dist)
  end
end

#add_measure_from_path(instance_name, instance_display_name, path_to_measure) ⇒ Object

Add a measure to the workflow from a path. Inside the path it is expecting to have a measure.json file if not, the BCL gem is used to create the measure.json file.

Parameters:

  • path_to_measure (String) —

    This is the local path to the measure directory, relative or absolute. It is used when zipping up all the measures.

Returns:

  • (Object) —

    Returns the measure that was added as an OpenStudio::AnalysisWorkflowStep object



28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
# File 'lib/openstudio/analysis/workflow.rb', line 28

def add_measure_from_path(instance_name, instance_display_name, path_to_measure)
  measure_filename = 'measure.rb'
  if File.exist?(path_to_measure) && File.file?(path_to_measure)
    measure_filename = File.basename(path_to_measure)
    path_to_measure = File.dirname(path_to_measure)
  end

  if Dir.exist?(path_to_measure) && File.directory?(path_to_measure)
    # Watch out for namespace conflicts (use ::BCL)
    b = ::BCL::ComponentMethods.new
    measure_hash = nil
    unless File.exist?(File.join(path_to_measure, 'measure.json'))
      measure_hash = b.parse_measure_file(nil, File.join(path_to_measure, measure_filename))
      File.open(File.join(path_to_measure, 'measure.json'), 'w') { |f| f << JSON.pretty_generate(measure_hash) }
      warn("measure.json not found in #{path_to_measure}, will parse measure file using BCL gem")
    end

    if measure_hash.nil? && File.exist?(File.join(path_to_measure, 'measure.json'))
      measure_hash = JSON.parse(File.read(File.join(path_to_measure, 'measure.json')), symbolize_names: true)
    elsif measure_hash.nil?
      fail 'measure.json was not found and was not automatically created'
    end

    add_measure(instance_name, instance_display_name, path_to_measure, measure_hash)
  else
    fail "could not find measure to add to workflow #{path_to_measure}"
  end

  @items.last
end

#all_variables ⇒ Array

Return all the variables in the analysis as an array. The list that is returned is read only.

Returns:

  • (Array) —

    All variables in the workflow



143
144
145
# File 'lib/openstudio/analysis/workflow.rb', line 143

def all_variables
  @items.map(&:variables).flatten
end

#clear ⇒ Object

Remove all the items in the workflow



17
18
19
# File 'lib/openstudio/analysis/workflow.rb', line 17

def clear
  @items.clear
end

#each ⇒ Object

Iterate over all the WorkflowItems



128
129
130
# File 'lib/openstudio/analysis/workflow.rb', line 128

def each
  @items.each { |i| yield i }
end

#find_measure(instance_name) ⇒ Object Also known as: find_workflow_step

Find the measure by its instance name

Returns:

  • (Object) —

    The WorkflowStep with the instance_name



135
136
137
# File 'lib/openstudio/analysis/workflow.rb', line 135

def find_measure(instance_name)
  @items.find { |i| i.name == instance_name }
end

#to_hash(version = 1) ⇒ Object

Save the workflow to a hash object



148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
165
# File 'lib/openstudio/analysis/workflow.rb', line 148

def to_hash(version = 1)
  h = nil
  if version == 1
    arr = []
    @items.each_with_index do |item, index|
      temp_h = item.to_hash(version)
      temp_h[:workflow_index] = index

      arr << temp_h
    end

    h = arr
  else
    fail "Version #{version} not yet implemented for to_hash"
  end

  h
end

#to_json(version = 1) ⇒ String

Save the workflow to a JSON string

Returns:

  • (String) —

    JSON formatted string



170
171
172
173
174
175
176
# File 'lib/openstudio/analysis/workflow.rb', line 170

def to_json(version = 1)
  if version == 1
    JSON.pretty_generate(to_hash(version))
  else
    fail "Version #{version} not yet implemented for to_json"
  end
end