Class: OpenStudio::Analysis::ServerApi
- Inherits:
-
Object
- Object
- OpenStudio::Analysis::ServerApi
- Defined in:
- lib/openstudio/analysis/server_api.rb
Instance Attribute Summary collapse
-
#hostname ⇒ Object
readonly
Returns the value of attribute hostname.
Instance Method Summary collapse
- #analysis_dencity_json(analysis_id) ⇒ Object
- #datapoint_dencity(datapoint_id) ⇒ Object
- #delete_all ⇒ Object
- #delete_project(id) ⇒ Object
-
#download_database(save_directory = '.') ⇒ Object
Download a MongoDB Snapshot.
- #download_dataframe(analysis_id, format = 'rdata', save_directory = '.') ⇒ Object
- #download_datapoint(datapoint_id, save_directory = '.') ⇒ Object
- #download_datapoint_dencity_jsons(analysis_id, save_directory = '.') ⇒ Object
- #download_datapoint_jsons(analysis_id, save_directory = '.') ⇒ Object
- #download_datapoint_reports(datapoint_id, save_directory = '.') ⇒ Object
- #download_datapoints_reports(analysis_id, save_directory = '.') ⇒ Object
- #download_dencity_json(analysis_id, save_directory = '.') ⇒ Object
- #download_variables(analysis_id, format = 'rdata', save_directory = '.') ⇒ Object
- #get_analyses(project_id) ⇒ Object
- #get_analyses_detailed(project_id) ⇒ Object
-
#get_analysis(analysis_id) ⇒ Object
return the entire analysis JSON.
-
#get_analysis_results(analysis_id) ⇒ Object
return the data point results in JSON format.
-
#get_analysis_status(analysis_id, analysis_type) ⇒ Object
Check the status of the simulation.
- #get_analysis_status_and_json(analysis_id, analysis_type) ⇒ Object
-
#get_datapoint(data_point_id) ⇒ Object
Return the JSON (Full) of the datapoint.
- #get_datapoint_status(analysis_id, filter = nil) ⇒ Object
- #get_project_ids ⇒ Object
- #get_projects ⇒ Object
-
#initialize(options = {}) ⇒ ServerApi
constructor
A new instance of ServerApi.
- #kill_all_analyses ⇒ Object
- #kill_analysis(analysis_id) ⇒ Object
- #new_analysis(project_id, options) ⇒ Object
- #new_project(options = {}) ⇒ Object
- #run_analysis(analysis_id, options) ⇒ Object
- #run_analysis_detailed(formulation_filename, analysis_zip_filename, analysis_type, allow_multiple_jobs, server_as_worker, run_data_point_filename) ⇒ Object
- #run_lhs(formulation_filename, analysis_zip_filename) ⇒ Object
-
#run_rgenoud(formulation_filename, analysis_zip_filename, _number_of_generations) ⇒ Object
creates a new analysis and runs rgenoud optimization - number of generations isn't used right now.
-
#run_single_model(formulation_filename, analysis_zip_filename, run_data_point_filename = 'run_openstudio_workflow_monthly.rb') ⇒ Object
create a new analysis and run a single model.
- #upload_datapoint(analysis_id, options) ⇒ Object
- #upload_datapoints(analysis_id, options) ⇒ Object
Constructor Details
#initialize(options = {}) ⇒ ServerApi
Returns a new instance of ServerApi.
8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 |
# File 'lib/openstudio/analysis/server_api.rb', line 8 def initialize( = {}) defaults = { hostname: 'http://localhost:8080' } = defaults.merge() @logger = ::Logger.new('faraday.log') @hostname = [:hostname] fail 'no host defined for server api class' if @hostname.nil? # todo: add support for the proxy # create connection with basic capabilities @conn = Faraday.new(url: @hostname) do |faraday| faraday.request :url_encoded # form-encode POST params faraday.use Faraday::Response::Logger, @logger # faraday.response @logger # log requests to STDOUT faraday.adapter Faraday.default_adapter # make requests with Net::HTTP end # create connection to server api with multipart capabilities @conn_multipart = Faraday.new(url: @hostname) do |faraday| faraday.request :multipart faraday.request :url_encoded # form-encode POST params faraday.use Faraday::Response::Logger, @logger # faraday.response :logger # log requests to STDOUT faraday.adapter Faraday.default_adapter # make requests with Net::HTTP end end |
Instance Attribute Details
#hostname ⇒ Object (readonly)
Returns the value of attribute hostname.
6 7 8 |
# File 'lib/openstudio/analysis/server_api.rb', line 6 def hostname @hostname end |
Instance Method Details
#analysis_dencity_json(analysis_id) ⇒ Object
336 337 338 339 340 341 342 343 344 345 346 |
# File 'lib/openstudio/analysis/server_api.rb', line 336 def analysis_dencity_json(analysis_id) # Return the hash of the dencity format for the analysis dencity = nil resp = @conn.get "/analyses/#{analysis_id}/dencity.json" if resp.status == 200 dencity = JSON.parse resp.body, symbolize_names: true end dencity end |
#datapoint_dencity(datapoint_id) ⇒ Object
324 325 326 327 328 329 330 331 332 333 334 |
# File 'lib/openstudio/analysis/server_api.rb', line 324 def datapoint_dencity(datapoint_id) # Return the JSON (Full) of the datapoint data_point = nil resp = @conn.get "/data_points/#{datapoint_id}/dencity.json" if resp.status == 200 data_point = JSON.parse resp.body, symbolize_names: true end data_point end |
#delete_all ⇒ Object
69 70 71 72 73 74 75 76 77 78 79 |
# File 'lib/openstudio/analysis/server_api.rb', line 69 def delete_all ids = get_project_ids puts "deleting projects with IDs: #{ids}" success = true ids.each do |id| r = delete_project id success = false if r == false end success end |
#delete_project(id) ⇒ Object
55 56 57 58 59 60 61 62 63 64 65 66 67 |
# File 'lib/openstudio/analysis/server_api.rb', line 55 def delete_project(id) deleted = false response = @conn.delete "/projects/#{id}.json" if response.status == 204 puts "Successfully deleted project #{id}" deleted = true else puts "ERROR deleting project #{id}" deleted = false end deleted end |
#download_database(save_directory = '.') ⇒ Object
Download a MongoDB Snapshot. This database can get large. For 13,000 simulations with DEnCity reporting, the size is around 325MB
267 268 269 270 271 272 273 274 275 276 277 278 279 280 281 282 283 284 285 |
# File 'lib/openstudio/analysis/server_api.rb', line 267 def download_database(save_directory = '.') downloaded = false file_path_and_name = nil response = @conn.get do |r| r.url '/admin/backup_database?full_backup=true' r..timeout = 3600 # 60 minutes end if response.status == 200 filename = response['content-disposition'].match(/filename=(\"?)(.+)\1/)[2] downloaded = true file_path_and_name = "#{save_directory}/#{filename}" puts "File #{filename} already exists, overwriting" if File.exist?(file_path_and_name) File.open(file_path_and_name, 'wb') { |f| f << response.body } end [downloaded, file_path_and_name] end |
#download_dataframe(analysis_id, format = 'rdata', save_directory = '.') ⇒ Object
206 207 208 209 210 211 212 213 214 215 216 217 218 219 220 221 222 223 224 225 226 227 |
# File 'lib/openstudio/analysis/server_api.rb', line 206 def download_dataframe(analysis_id, format = 'rdata', save_directory = '.') downloaded = false file_path_and_name = nil response = @conn.get do |r| r.url "/analyses/#{analysis_id}/download_data.#{format}?export=true" r..timeout = 3600 # 60 minutes end if response.status == 200 filename = response['content-disposition'].match(/filename=(\"?)(.+)\1/)[2] downloaded = true file_path_and_name = "#{save_directory}/#{filename}" puts "File #{filename} already exists, overwriting" if File.exist?(file_path_and_name) if format == 'rdata' File.open(file_path_and_name, 'wb') { |f| f << response.body } else File.open(file_path_and_name, 'w') { |f| f << response.body } end end [downloaded, file_path_and_name] end |
#download_datapoint(datapoint_id, save_directory = '.') ⇒ Object
249 250 251 252 253 254 255 256 257 258 259 260 261 262 263 |
# File 'lib/openstudio/analysis/server_api.rb', line 249 def download_datapoint(datapoint_id, save_directory = '.') downloaded = false file_path_and_name = nil response = @conn.get "/data_points/#{datapoint_id}/download" if response.status == 200 filename = response['content-disposition'].match(/filename=(\"?)(.+)\1/)[2] downloaded = true file_path_and_name = "#{save_directory}/#{filename}" puts "File #{filename} already exists, overwriting" if File.exist?(file_path_and_name) File.open(file_path_and_name, 'wb') { |f| f << response.body } end [downloaded, file_path_and_name] end |
#download_datapoint_dencity_jsons(analysis_id, save_directory = '.') ⇒ Object
355 356 357 358 359 360 361 362 363 364 |
# File 'lib/openstudio/analysis/server_api.rb', line 355 def download_datapoint_dencity_jsons(analysis_id, save_directory = '.') # get the list of all the datapoints dps = get_datapoint_status(analysis_id) dps.each do |dp| if dp[:status] == 'completed' dp_h = datapoint_dencity(dp[:_id]) File.open("#{save_directory}/data_point_#{dp[:_id]}_dencity.json", 'w') { |f| f << JSON.pretty_generate(dp_h) } end end end |
#download_datapoint_jsons(analysis_id, save_directory = '.') ⇒ Object
313 314 315 316 317 318 319 320 321 322 |
# File 'lib/openstudio/analysis/server_api.rb', line 313 def download_datapoint_jsons(analysis_id, save_directory = '.') # get the list of all the datapoints dps = get_datapoint_status(analysis_id) dps.each do |dp| if dp[:status] == 'completed' dp_h = get_datapoint(dp[:_id]) File.open("#{save_directory}/data_point_#{dp[:_id]}.json", 'w') { |f| f << JSON.pretty_generate(dp_h) } end end end |
#download_datapoint_reports(datapoint_id, save_directory = '.') ⇒ Object
287 288 289 290 291 292 293 294 295 296 297 298 299 300 301 |
# File 'lib/openstudio/analysis/server_api.rb', line 287 def download_datapoint_reports(datapoint_id, save_directory = '.') downloaded = false file_path_and_name = nil response = @conn.get "/data_points/#{datapoint_id}/download_reports" if response.status == 200 filename = response['content-disposition'].match(/filename=(\"?)(.+)\1/)[2] downloaded = true file_path_and_name = "#{save_directory}/#{filename}" puts "File #{filename} already exists, overwriting" if File.exist?(file_path_and_name) File.open(file_path_and_name, 'wb') { |f| f << response.body } end [downloaded, file_path_and_name] end |
#download_datapoints_reports(analysis_id, save_directory = '.') ⇒ Object
303 304 305 306 307 308 309 310 311 |
# File 'lib/openstudio/analysis/server_api.rb', line 303 def download_datapoints_reports(analysis_id, save_directory = '.') # get the list of all the datapoints dps = get_datapoint_status(analysis_id) dps.each do |dp| if dp[:status] == 'completed' download_datapoint_reports(dp[:_id], save_directory) end end end |
#download_dencity_json(analysis_id, save_directory = '.') ⇒ Object
348 349 350 351 352 353 |
# File 'lib/openstudio/analysis/server_api.rb', line 348 def download_dencity_json(analysis_id, save_directory = '.') a_h = analysis_dencity_json(analysis_id) if a_h File.open("#{save_directory}/analysis_#{analysis_id}_dencity.json", 'w') { |f| f << JSON.pretty_generate(a_h) } end end |
#download_variables(analysis_id, format = 'rdata', save_directory = '.') ⇒ Object
229 230 231 232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 247 |
# File 'lib/openstudio/analysis/server_api.rb', line 229 def download_variables(analysis_id, format = 'rdata', save_directory = '.') downloaded = false file_path_and_name = nil response = @conn.get "/analyses/#{analysis_id}/variables/download_variables.#{format}" if response.status == 200 filename = response['content-disposition'].match(/filename=(\"?)(.+)\1/)[2] downloaded = true file_path_and_name = "#{save_directory}/#{filename}" puts "File #{filename} already exists, overwriting" if File.exist?(file_path_and_name) if format == 'rdata' File.open(file_path_and_name, 'wb') { |f| f << response.body } else File.open(file_path_and_name, 'w') { |f| f << response.body } end end [downloaded, file_path_and_name] end |
#get_analyses(project_id) ⇒ Object
108 109 110 111 112 113 114 115 116 117 118 119 120 121 |
# File 'lib/openstudio/analysis/server_api.rb', line 108 def get_analyses(project_id) analysis_ids = [] response = @conn.get "/projects/#{project_id}.json" if response.status == 200 analyses = JSON.parse(response.body, symbolize_names: true, max_nesting: false) if analyses[:analyses] analyses[:analyses].each do |analysis| analysis_ids << analysis[:_id] end end end analysis_ids end |
#get_analyses_detailed(project_id) ⇒ Object
123 124 125 126 127 128 129 130 131 |
# File 'lib/openstudio/analysis/server_api.rb', line 123 def get_analyses_detailed(project_id) analyses = nil response = @conn.get "/projects/#{project_id}.json" if response.status == 200 analyses = JSON.parse(response.body, symbolize_names: true, max_nesting: false)[:analyses] end analyses end |
#get_analysis(analysis_id) ⇒ Object
return the entire analysis JSON
134 135 136 137 138 139 140 141 142 |
# File 'lib/openstudio/analysis/server_api.rb', line 134 def get_analysis(analysis_id) result = nil response = @conn.get "/analyses/#{analysis_id}.json" if response.status == 200 result = JSON.parse(response.body, symbolize_names: true, max_nesting: false)[:analysis] end result end |
#get_analysis_results(analysis_id) ⇒ Object
return the data point results in JSON format
195 196 197 198 199 200 201 202 203 204 |
# File 'lib/openstudio/analysis/server_api.rb', line 195 def get_analysis_results(analysis_id) analysis = nil response = @conn.get "/analyses/#{analysis_id}/analysis_data.json" if response.status == 200 analysis = JSON.parse(response.body, symbolize_names: true, max_nesting: false) end analysis end |
#get_analysis_status(analysis_id, analysis_type) ⇒ Object
Check the status of the simulation. Format should be: { analysis: { status: "completed", analysis_type: "batch_run" }, data_points: [ { _id: "bbd57e90-ce59-0131-35de-080027880ca6", status: "completed" } ] }
157 158 159 160 161 162 163 164 165 166 167 168 169 170 171 172 173 |
# File 'lib/openstudio/analysis/server_api.rb', line 157 def get_analysis_status(analysis_id, analysis_type) status = nil # sleep 2 # super cheesy---need to update how this works. Right now there is a good chance to get a # race condition when the analysis state changes. unless analysis_id.nil? resp = @conn.get "analyses/#{analysis_id}/status.json" if resp.status == 200 j = JSON.parse resp.body, symbolize_names: true if j && j[:analysis] && j[:analysis][:analysis_type] == analysis_type status = j[:analysis][:status] end end end status end |
#get_analysis_status_and_json(analysis_id, analysis_type) ⇒ Object
175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 |
# File 'lib/openstudio/analysis/server_api.rb', line 175 def get_analysis_status_and_json(analysis_id, analysis_type) status = nil j = nil # sleep 2 # super cheesy---need to update how this works. Right now there is a good chance to get a # race condition when the analysis state changes. unless analysis_id.nil? resp = @conn.get "analyses/#{analysis_id}/status.json" if resp.status == 200 j = JSON.parse resp.body, symbolize_names: true if j && j[:analysis] && j[:analysis][:analysis_type] == analysis_type status = j[:analysis][:status] end end end [status, j] end |
#get_datapoint(data_point_id) ⇒ Object
Return the JSON (Full) of the datapoint
576 577 578 579 580 581 582 583 584 585 |
# File 'lib/openstudio/analysis/server_api.rb', line 576 def get_datapoint(data_point_id) data_point = nil resp = @conn.get "/data_points/#{data_point_id}/show_full.json" if resp.status == 200 data_point = JSON.parse resp.body, symbolize_names: true end data_point end |
#get_datapoint_status(analysis_id, filter = nil) ⇒ Object
555 556 557 558 559 560 561 562 563 564 565 566 567 568 569 570 571 572 573 |
# File 'lib/openstudio/analysis/server_api.rb', line 555 def get_datapoint_status(analysis_id, filter = nil) data_points = nil # get the status of all the entire analysis unless analysis_id.nil? if filter.nil? || filter == '' resp = @conn.get "analyses/#{analysis_id}/status.json" if resp.status == 200 data_points = JSON.parse(resp.body, symbolize_names: true)[:data_points] end else resp = @conn.get "#{@hostname}/analyses/#{analysis_id}/status.json", jobs: filter if resp.status == 200 data_points = JSON.parse(resp.body, symbolize_names: true)[:data_points] end end end data_points end |
#get_project_ids ⇒ Object
50 51 52 53 |
# File 'lib/openstudio/analysis/server_api.rb', line 50 def get_project_ids ids = get_projects ids.map { |project| project[:uuid] } end |
#get_projects ⇒ Object
37 38 39 40 41 42 43 44 45 46 47 48 |
# File 'lib/openstudio/analysis/server_api.rb', line 37 def get_projects response = @conn.get '/projects.json' projects_json = nil if response.status == 200 projects_json = JSON.parse(response.body, symbolize_names: true, max_nesting: false) else fail 'did not receive a 200 in get_projects' end projects_json end |
#kill_all_analyses ⇒ Object
541 542 543 544 545 546 547 548 549 550 551 552 553 |
# File 'lib/openstudio/analysis/server_api.rb', line 541 def kill_all_analyses project_ids = get_project_ids puts "List of projects ids are: #{project_ids}" project_ids.each do |project_id| analysis_ids = get_analyses(project_id) puts analysis_ids analysis_ids.each do |analysis_id| puts "Trying to kill #{analysis_id}" kill_analysis(analysis_id) end end end |
#kill_analysis(analysis_id) ⇒ Object
525 526 527 528 529 530 531 532 533 534 535 536 537 538 539 |
# File 'lib/openstudio/analysis/server_api.rb', line 525 def kill_analysis(analysis_id) analysis_action = { analysis_action: 'stop' } response = @conn.post do |req| req.url "analyses/#{analysis_id}/action.json" req.headers['Content-Type'] = 'application/json' req.body = analysis_action.to_json end if response.status == 200 puts "Killed analysis #{analysis_id}" else # raise "Could not kill the analysis with response of #{response.inspect}" end end |
#new_analysis(project_id, options) ⇒ Object
366 367 368 369 370 371 372 373 374 375 376 377 378 379 380 381 382 383 384 385 386 387 388 389 390 391 392 393 394 395 396 397 398 399 400 401 402 403 404 405 406 407 408 409 410 411 412 413 414 415 416 417 418 419 420 421 422 423 424 425 426 427 428 429 430 431 432 433 434 435 436 437 438 439 440 441 442 443 444 445 446 447 448 449 450 451 |
# File 'lib/openstudio/analysis/server_api.rb', line 366 def new_analysis(project_id, ) defaults = { analysis_name: nil, reset_uuids: false } = defaults.merge() fail 'No project id passed' if project_id.nil? formulation_json = nil if [:formulation_file] fail "No formulation exists #{[:formulation_file]}" unless File.exist?([:formulation_file]) formulation_json = JSON.parse(File.read([:formulation_file]), symbolize_names: true) end # read in the analysis id from the analysis.json file analysis_id = nil if formulation_json if [:reset_uuids] analysis_id = SecureRandom.uuid formulation_json[:analysis][:uuid] = analysis_id formulation_json[:analysis][:problem][:workflow].each do |wf| wf[:uuid] = SecureRandom.uuid if wf[:arguments] wf[:arguments].each do |arg| arg[:uuid] = SecureRandom.uuid end end if wf[:variables] wf[:variables].each do |var| var[:uuid] = SecureRandom.uuid var[:argument][:uuid] = SecureRandom.uuid if var[:argument] end end end else analysis_id = formulation_json[:analysis][:uuid] end # set the analysis name formulation_json[:analysis][:name] = "#{[:analysis_name]}" unless [:analysis_name].nil? else formulation_json = { analysis: } puts formulation_json analysis_id = SecureRandom.uuid formulation_json[:analysis][:uuid] = analysis_id end fail "No analysis id defined in analyis.json #{[:formulation_file]}" if analysis_id.nil? # save out this file to compare # File.open('formulation_merge.json', 'w') { |f| f << JSON.pretty_generate(formulation_json) } response = @conn.post do |req| req.url "projects/#{project_id}/analyses.json" req.headers['Content-Type'] = 'application/json' req.body = formulation_json.to_json end if response.status == 201 puts "asked to create analysis with #{analysis_id}" # puts resp.inspect analysis_id = JSON.parse(response.body)['_id'] puts "new analysis created with ID: #{analysis_id}" else fail 'Could not create new analysis' end # check if we need to upload the analysis zip file if [:upload_file] fail "upload file does not exist #{[:upload_file]}" unless File.exist?([:upload_file]) payload = { file: Faraday::UploadIO.new([:upload_file], 'application/zip') } response = @conn_multipart.post "analyses/#{analysis_id}/upload.json", payload do |req| req.[:timeout] = 1800 # seconds end if response.status == 201 puts 'Successfully uploaded ZIP file' else fail response.inspect end end analysis_id end |
#new_project(options = {}) ⇒ Object
81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 |
# File 'lib/openstudio/analysis/server_api.rb', line 81 def new_project( = {}) defaults = { project_name: "Project #{::Time.now.strftime('%Y-%m-%d %H:%M:%S')}" } = defaults.merge() project_id = nil # TODO: make this a display name and a machine name project_hash = { project: { name: "#{[:project_name]}" } } response = @conn.post do |req| req.url '/projects.json' req.headers['Content-Type'] = 'application/json' req.body = project_hash.to_json end if response.status == 201 project_id = JSON.parse(response.body)['_id'] puts "new project created with ID: #{project_id}" # grab the project id elsif response.status == 500 puts '500 Error' puts response.inspect end project_id end |
#run_analysis(analysis_id, options) ⇒ Object
506 507 508 509 510 511 512 513 514 515 516 517 518 519 520 521 522 523 |
# File 'lib/openstudio/analysis/server_api.rb', line 506 def run_analysis(analysis_id, ) defaults = { analysis_action: 'start', without_delay: false } = defaults.merge() puts "Run analysis is configured with #{.to_json}" response = @conn.post do |req| req.url "analyses/#{analysis_id}/action.json" req.headers['Content-Type'] = 'application/json' req.body = .to_json req.[:timeout] = 1800 # seconds end if response.status == 200 puts "Recieved request to run analysis #{analysis_id}" else fail 'Could not start the analysis' end end |
#run_analysis_detailed(formulation_filename, analysis_zip_filename, analysis_type, allow_multiple_jobs, server_as_worker, run_data_point_filename) ⇒ Object
689 690 691 692 693 694 695 696 697 698 699 700 701 702 703 704 705 706 707 708 709 710 711 712 713 714 715 716 717 718 719 720 721 722 723 724 725 726 727 728 729 |
# File 'lib/openstudio/analysis/server_api.rb', line 689 def run_analysis_detailed(formulation_filename, analysis_zip_filename, analysis_type, allow_multiple_jobs, server_as_worker, run_data_point_filename) = {} project_id = new_project() = { formulation_file: formulation_filename, upload_file: analysis_zip_filename, reset_uuids: true } analysis_id = new_analysis(project_id, ) server_as_worker = true if analysis_type == 'optim' || analysis_type == 'rgenoud' = { analysis_action: 'start', without_delay: false, analysis_type: analysis_type, allow_multiple_jobs: allow_multiple_jobs, use_server_as_worker: server_as_worker, simulate_data_point_filename: 'simulate_data_point.rb', run_data_point_filename: run_data_point_filename } run_analysis(analysis_id, ) # If the analysis is LHS, then go ahead and run batch run because there is # no explicit way to tell the system to do it if analysis_type == 'lhs' || analysis_type == 'preflight' || analysis_type == 'single_run' = { analysis_action: 'start', without_delay: false, analysis_type: 'batch_run', allow_multiple_jobs: allow_multiple_jobs, use_server_as_worker: server_as_worker, simulate_data_point_filename: 'simulate_data_point.rb', run_data_point_filename: run_data_point_filename } run_analysis(analysis_id, ) end analysis_id end |
#run_lhs(formulation_filename, analysis_zip_filename) ⇒ Object
653 654 655 656 657 658 659 660 661 662 663 664 665 666 667 668 669 670 671 672 673 674 675 676 677 678 679 680 681 682 683 684 685 686 687 |
# File 'lib/openstudio/analysis/server_api.rb', line 653 def run_lhs(formulation_filename, analysis_zip_filename) = {} project_id = new_project() = { formulation_file: formulation_filename, upload_file: analysis_zip_filename, reset_uuids: true } analysis_id = new_analysis(project_id, ) = { analysis_action: 'start', without_delay: false, analysis_type: 'lhs', allow_multiple_jobs: true, use_server_as_worker: true, simulate_data_point_filename: 'simulate_data_point.rb', run_data_point_filename: 'run_openstudio_workflow_monthly.rb' } run_analysis(analysis_id, ) = { analysis_action: 'start', without_delay: false, # run in background analysis_type: 'batch_run', allow_multiple_jobs: true, use_server_as_worker: true, simulate_data_point_filename: 'simulate_data_point.rb', run_data_point_filename: 'run_openstudio_workflow_monthly.rb' } run_analysis(analysis_id, ) analysis_id end |
#run_rgenoud(formulation_filename, analysis_zip_filename, _number_of_generations) ⇒ Object
creates a new analysis and runs rgenoud optimization - number of generations isn't used right now
628 629 630 631 632 633 634 635 636 637 638 639 640 641 642 643 644 645 646 647 648 649 650 651 |
# File 'lib/openstudio/analysis/server_api.rb', line 628 def run_rgenoud(formulation_filename, analysis_zip_filename, _number_of_generations) = {} project_id = new_project() = { formulation_file: formulation_filename, upload_file: analysis_zip_filename, reset_uuids: true } analysis_id = new_analysis(project_id, ) = { analysis_action: 'start', without_delay: false, analysis_type: 'rgenoud', allow_multiple_jobs: true, use_server_as_worker: true, simulate_data_point_filename: 'simulate_data_point.rb', run_data_point_filename: 'run_openstudio_workflow_monthly.rb' } run_analysis(analysis_id, ) analysis_id end |
#run_single_model(formulation_filename, analysis_zip_filename, run_data_point_filename = 'run_openstudio_workflow_monthly.rb') ⇒ Object
create a new analysis and run a single model
590 591 592 593 594 595 596 597 598 599 600 601 602 603 604 605 606 607 608 609 610 611 612 613 614 615 616 617 618 619 620 621 622 623 624 625 |
# File 'lib/openstudio/analysis/server_api.rb', line 590 def run_single_model(formulation_filename, analysis_zip_filename, run_data_point_filename = 'run_openstudio_workflow_monthly.rb') = {} project_id = new_project() = { formulation_file: formulation_filename, upload_file: analysis_zip_filename, reset_uuids: true } analysis_id = new_analysis(project_id, ) # Force this to run in the foreground for now until we can deal with checing the 'analysis state of various anlaysis' = { analysis_action: 'start', without_delay: true, # run this in the foreground analysis_type: 'single_run', allow_multiple_jobs: true, use_server_as_worker: true, simulate_data_point_filename: 'simulate_data_point.rb', run_data_point_filename: run_data_point_filename } run_analysis(analysis_id, ) = { analysis_action: 'start', without_delay: false, # run in background analysis_type: 'batch_run', allow_multiple_jobs: true, use_server_as_worker: true, simulate_data_point_filename: 'simulate_data_point.rb', run_data_point_filename: run_data_point_filename } run_analysis(analysis_id, ) analysis_id end |
#upload_datapoint(analysis_id, options) ⇒ Object
453 454 455 456 457 458 459 460 461 462 463 464 465 466 467 468 469 470 471 472 473 474 475 476 477 478 479 480 |
# File 'lib/openstudio/analysis/server_api.rb', line 453 def upload_datapoint(analysis_id, ) defaults = { reset_uuids: false } = defaults.merge() fail 'No analysis id passed' if analysis_id.nil? fail 'No datapoints file passed to new_analysis' unless [:datapoint_file] fail "No datapoints_file exists #{[:datapoint_file]}" unless File.exist?([:datapoint_file]) dp_hash = JSON.parse(File.open([:datapoint_file]).read, symbolize_names: true) if [:reset_uuids] dp_hash[:analysis_uuid] = analysis_id dp_hash[:uuid] = SecureRandom.uuid end # merge in the analysis_id as it has to be what is in the database response = @conn.post do |req| req.url "analyses/#{analysis_id}/data_points.json" req.headers['Content-Type'] = 'application/json' req.body = dp_hash.to_json end if response.status == 201 puts "new datapoints created for analysis #{analysis_id}" else fail "could not create new datapoints #{response.body}" end end |
#upload_datapoints(analysis_id, options) ⇒ Object
482 483 484 485 486 487 488 489 490 491 492 493 494 495 496 497 498 499 500 501 502 503 504 |
# File 'lib/openstudio/analysis/server_api.rb', line 482 def upload_datapoints(analysis_id, ) defaults = {} = defaults.merge() fail 'No analysis id passed' if analysis_id.nil? fail 'No datapoints file passed to new_analysis' unless [:datapoints_file] fail "No datapoints_file exists #{[:datapoints_file]}" unless File.exist?([:datapoints_file]) dp_hash = JSON.parse(File.open([:datapoints_file]).read, symbolize_names: true) # merge in the analysis_id as it has to be what is in the database response = @conn.post do |req| req.url "analyses/#{analysis_id}/data_points/batch_upload.json" req.headers['Content-Type'] = 'application/json' req.body = dp_hash.to_json end if response.status == 201 puts "new datapoints created for analysis #{analysis_id}" else fail "could not create new datapoints #{response.body}" end end |